'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var inversify = require('inversify');
var React = require('react');
var React__default = _interopDefault(React);
var isWhat = require('is-what');

var config = {
  debug: false
};
function configure(configuration) {
  Object.assign(config, configuration);
}

function _defineProperties(target, props) {
  for (var i = 0; i < props.length; i++) {
    var descriptor = props[i];
    descriptor.enumerable = descriptor.enumerable || false;
    descriptor.configurable = true;
    if ("value" in descriptor) descriptor.writable = true;
    Object.defineProperty(target, descriptor.key, descriptor);
  }
}

function _createClass(Constructor, protoProps, staticProps) {
  if (protoProps) _defineProperties(Constructor.prototype, protoProps);
  if (staticProps) _defineProperties(Constructor, staticProps);
  return Constructor;
}

function _extends() {
  _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  return _extends.apply(this, arguments);
}

function _unsupportedIterableToArray(o, minLen) {
  if (!o) return;
  if (typeof o === "string") return _arrayLikeToArray(o, minLen);
  var n = Object.prototype.toString.call(o).slice(8, -1);
  if (n === "Object" && o.constructor) n = o.constructor.name;
  if (n === "Map" || n === "Set") return Array.from(o);
  if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen);
}

function _arrayLikeToArray(arr, len) {
  if (len == null || len > arr.length) len = arr.length;

  for (var i = 0, arr2 = new Array(len); i < len; i++) arr2[i] = arr[i];

  return arr2;
}

function _createForOfIteratorHelperLoose(o, allowArrayLike) {
  var it = typeof Symbol !== "undefined" && o[Symbol.iterator] || o["@@iterator"];
  if (it) return (it = it.call(o)).next.bind(it);

  if (Array.isArray(o) || (it = _unsupportedIterableToArray(o)) || allowArrayLike && o && typeof o.length === "number") {
    if (it) o = it;
    var i = 0;
    return function () {
      if (i >= o.length) return {
        done: true
      };
      return {
        done: false,
        value: o[i++]
      };
    };
  }

  throw new TypeError("Invalid attempt to iterate non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.");
}

function debug() {
  if (config.debug) {
    var _console;

    for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    (_console = console).debug.apply(_console, ['[react-service-locator]'].concat(args));
  }
}

var defaultServices = /*#__PURE__*/new Map();
var callbacks = [];
function registerDefaultService(token, value) {
  debug('Registering default service', token);
  defaultServices.set(token, value);

  for (var _iterator = _createForOfIteratorHelperLoose(callbacks), _step; !(_step = _iterator()).done;) {
    var cb = _step.value;
    debug("Notifying a cb " + cb.name + " about", token);
    cb(token, value);
  }
}
function addDefaultServiceRegisteredCallback(cb) {
  if (callbacks.indexOf(cb) < 0) {
    debug('Adding a cb', cb.name);
    callbacks.push(cb);
  }
}
function removeDefaultServiceRegisteredCallback(cb) {
  debug('Removing cb', cb.name);
  callbacks.splice(callbacks.indexOf(cb), 1);
}

var Service = function Service(options) {
  if (options === void 0) {
    options = {};
  }

  var _options = options,
      provide = _options.provide;
  var _options2 = options,
      _options2$scope = _options2.scope,
      scope = _options2$scope === void 0 ? 'singleton' : _options2$scope; // eslint-disable-next-line @typescript-eslint/ban-types

  return function (constructor) {
    if (typeof provide === 'undefined') {
      provide = constructor;
    }

    inversify.injectable()(constructor);
    registerDefaultService(provide, {
      cls: constructor,
      scope: scope
    });
  };
};

/**
 * @deprecated since version 1.4. use @Service
 */

var Injectable = Service;

var notInitialized = /*#__PURE__*/Symbol('notInitialized');
var StatefulService = /*#__PURE__*/function () {
  function StatefulService(initialState) {
    this.listeners = [];
    this._state = notInitialized;

    if (typeof initialState !== 'undefined') {
      this._state = initialState;
    }
  }

  var _proto = StatefulService.prototype;

  _proto.assertIsInitialized = function assertIsInitialized(d) {
    if (d === notInitialized) {
      throw new Error('State has not been initialized.');
    }
  }
  /**
   * A getter for the internal state. It will return the same internal state
   * object (=== is true), but typed as recursively immutable.
   */
  ;

  _proto.setState = function setState(newState) {
    this.assertIsInitialized(this._state);

    if (isWhat.isPlainObject(this._state) && isWhat.isPlainObject(newState)) {
      this._state = _extends({}, this._state, newState);
    } else {
      this._state = newState;
    }

    for (var _iterator = _createForOfIteratorHelperLoose(this.listeners), _step; !(_step = _iterator()).done;) {
      var listener = _step.value;
      listener();
    }
  };

  _proto.addListener = function addListener(listener) {
    this.listeners.push(listener);
  };

  _proto.removeListener = function removeListener(listener) {
    this.listeners.splice(this.listeners.indexOf(listener), 1);
  };

  _createClass(StatefulService, [{
    key: "state",
    get: function get() {
      this.assertIsInitialized(this._state);
      return this._state;
    }
    /**
     * Can be used to set state without triggering re-renders.
     * Should only be used in constructors this way.
     */
    ,
    set: function set(state) {
      this._state = state;
    }
  }]);

  return StatefulService;
}();

function isPrimitive(v) {
  // https://developer.mozilla.org/en-US/docs/Glossary/Primitive
  return typeof v === 'string' || typeof v === 'number' || typeof v === 'bigint' || typeof v === 'boolean' || typeof v === 'undefined' || typeof v === 'symbol' || v === null;
}

function shallowCompareArray(a, b) {
  return a.length === b.length && a.every(function (v, i) {
    return Object.is(v, b[i]);
  });
}

function shallowCompare(a, b) {
  if (typeof a !== typeof b || isPrimitive(a) || isPrimitive(b)) {
    return false;
  }

  if (Array.isArray(a)) {
    return Array.isArray(b) && shallowCompareArray(a, b);
  }

  if (typeof a === 'object') {
    return typeof b === 'object' && shallowCompareArray(Object.values(a), Object.values(b));
  }

  return false;
}

var ServiceLocatorContext = /*#__PURE__*/React__default.createContext(null);

function useGetService(token) {
  var container = React.useContext(ServiceLocatorContext);

  if (!container) {
    throw new Error('No DI container found.');
  }

  debug('Getting service with token', token);

  var _useRef = React.useRef(container.get(token)),
      service = _useRef.current;

  return service;
}

function useForceUpdate() {
  // https://reactjs.org/docs/hooks-faq.html#is-there-something-like-forceupdate
  var _useReducer = React.useReducer(function (x) {
    return x + 1;
  }, 0),
      forceUpdate = _useReducer[1];

  return forceUpdate;
}

function useService(token, depsFn) {
  var service = useGetService(token);
  var forceUpdate = useForceUpdate();
  React.useEffect(function () {
    if (service instanceof StatefulService) {
      var finalDepsFn = depsFn != null ? depsFn : function (service) {
        return service.state;
      };
      var compareFn = typeof depsFn === 'undefined' ? Object.is : shallowCompare;
      var lastValue = finalDepsFn(service);

      var listener = function listener() {
        var newValue = finalDepsFn(service);
        var valuesAreEqual = compareFn(lastValue, newValue);
        lastValue = newValue;

        if (!valuesAreEqual) {
          forceUpdate();
        }
      };

      service.addListener(listener);
      return function () {
        service.removeListener(listener);
      };
    } // eslint-disable-next-line react-hooks/exhaustive-deps

  }, [service]);
  return service;
}

function autoCompare(a, b) {
  if (typeof a !== typeof b) {
    return false;
  }

  if (isPrimitive(a)) {
    return isPrimitive(b) && Object.is(a, b);
  }

  return shallowCompare(a, b);
}

function useServiceSelector(token, selectorFn, compareFn) {
  var service = useGetService(token);
  var forceUpdate = useForceUpdate();

  var _useState = React.useState(function () {
    return {
      current: selectorFn(service)
    };
  }),
      lastValueRef = _useState[0];

  React.useEffect(function () {
    if (service instanceof StatefulService) {
      var finalCompareFn = typeof compareFn === 'undefined' ? autoCompare : compareFn;

      var listener = function listener() {
        var newValue = selectorFn(service);

        if (!finalCompareFn(lastValueRef.current, newValue)) {
          lastValueRef.current = newValue;
          forceUpdate();
        }
      };

      service.addListener(listener);
      return function () {
        service.removeListener(listener);
      };
    } // eslint-disable-next-line react-hooks/exhaustive-deps

  }, [service]);
  return lastValueRef.current;
}

function createStatefulServiceHookAlias(token) {
  return function (depsFn) {
    return useService(token, depsFn);
  };
}

function createContainer() {
  return new inversify.Container({
    skipBaseClassChecks: true,
    autoBindInjectable: false
  });
}

var ServiceContainer = function ServiceContainer(_ref) {
  var _ref$services = _ref.services,
      services = _ref$services === void 0 ? [] : _ref$services,
      children = _ref.children;
  var parentContainer = React.useContext(ServiceLocatorContext);
  var defaultServiceRegisteredCallback = React.useRef();
  var container = React.useMemo(function () {
    // https://github.com/inversify/InversifyJS/blob/master/wiki/hierarchical_di.md
    var isRoot = parentContainer === null;
    var container = isRoot ? createContainer() : // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
    parentContainer.createChild();
    var doneTokens = [];

    var registerServicesOnContainer = function registerServicesOnContainer() {
      var finalServices = services.slice();

      if (isRoot) {
        // only automatically register decorated services on root service containers
        defaultServices.forEach(function (_ref2, token) {
          var cls = _ref2.cls,
              scope = _ref2.scope;
          finalServices.unshift({
            provide: token,
            useClass: cls,
            scope: scope
          });
        });
      }

      var nRegistered = 0;

      for (var _iterator = _createForOfIteratorHelperLoose(finalServices.reverse()), _step; !(_step = _iterator()).done;) {
        var provider = _step.value;
        var finalProvider = void 0;

        if (typeof provider === 'function') {
          finalProvider = {
            provide: provider,
            useClass: provider,
            scope: 'singleton'
          };
        } else {
          finalProvider = provider;
        }

        var _finalProvider = finalProvider,
            provide = _finalProvider.provide;

        if (doneTokens.includes(provide)) {
          continue;
        }

        nRegistered++;
        doneTokens.push(provide);
        var scope = 'singleton';

        if ('scope' in finalProvider && finalProvider.scope) {
          scope = finalProvider.scope;
        }

        if ('useClass' in finalProvider && finalProvider.useClass) {
          var binding = container.bind(provide).to(finalProvider.useClass);

          if (scope === 'singleton') {
            binding.inSingletonScope();
          }
        } else if ('useFactory' in finalProvider && finalProvider.useFactory) {
          var _binding = container.bind(provide).toDynamicValue(finalProvider.useFactory);

          if (scope === 'singleton') {
            _binding.inSingletonScope();
          }
        } else if ('useValue' in finalProvider && finalProvider.useValue) {
          container.bind(provide).toConstantValue(finalProvider.useValue);
        } else {
          throw new Error('Unable to determine how to register provider.');
        }
      }

      debug("Registered " + nRegistered + " services on container.");
    };

    registerServicesOnContainer();

    defaultServiceRegisteredCallback.current = function defaultServiceRegisteredCallback() {
      registerServicesOnContainer();
    };

    addDefaultServiceRegisteredCallback(defaultServiceRegisteredCallback.current);
    return container; // eslint-disable-next-line react-hooks/exhaustive-deps
  }, [parentContainer]);
  React.useEffect(function () {
    return function () {
      if (defaultServiceRegisteredCallback.current) {
        removeDefaultServiceRegisteredCallback(defaultServiceRegisteredCallback.current);
      }
    };
  }, [parentContainer]);
  return React__default.createElement(ServiceLocatorContext.Provider, {
    value: container
  }, children);
};

Object.defineProperty(exports, 'Inject', {
  enumerable: true,
  get: function () {
    return inversify.inject;
  }
});
exports.Injectable = Injectable;
exports.Service = Service;
exports.ServiceContainer = ServiceContainer;
exports.StatefulService = StatefulService;
exports.configure = configure;
exports.createStatefulServiceHookAlias = createStatefulServiceHookAlias;
exports.useService = useService;
exports.useServiceSelector = useServiceSelector;
//# sourceMappingURL=react-service-locator.cjs.development.js.map
